package org.artifactory.hqc.cleanup;

import lombok.extern.slf4j.Slf4j;
import org.artifactory.api.context.ContextHelper;
import org.artifactory.hqc.service.HeavyQueryCacheService;
import org.artifactory.schedule.JobCommand;
import org.artifactory.schedule.JobName;
import org.artifactory.schedule.QuartzCommand;
import org.artifactory.schedule.TaskUser;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;

/**
 * @author Andrey Tashlitsky
 */
@Slf4j
@JobCommand(jobName = JobName.HEAVY_QUERY_CACHE_CLEANUP,
        description = "Heavy query cache DB table cleanup job",
        singleton = true,
        schedulerUser = TaskUser.SYSTEM)
public class HeavyQueryCacheCleanupJob extends QuartzCommand {

    @Override
    protected void onExecute(JobExecutionContext jobExecutionContext) throws JobExecutionException {
        log.debug("Heavy query cache DB table cleanup job has started at {}", jobExecutionContext.getFireTime());
        ContextHelper.get().beanForType(HeavyQueryCacheService.class).runCleanup();
        log.debug("Heavy query cache DB table cleanup job has finished. Job run time: {}, Next run at: {}",
                jobExecutionContext.getJobRunTime(), jobExecutionContext.getNextFireTime());
    }
}
