package org.artifactory.metadata.service.indexer;


import org.artifactory.async.Async;
import org.artifactory.metadata.service.MetadataEventService;
import org.artifactory.repo.RepoPath;

/**
 * An internal interface for exposing MDS "real time" indexer abilities to the event base indexer.
 *
 * @author BarakH
 */
public interface InternalMetadataServiceIndexHandler {

    /**
     * @return True if the real time Metadata index is enabled and the package type of the given repository is
     * supported by the real time indexer.
     */
    boolean isPackageTypeSupported(String repoPath);

    /**
     * Inject manually to remain the current functionality in the {@link MetadataEventService#init}.
     */
    void setMetadataEventService(MetadataEventService metadataEventService);


    /**
     * Submit internal path deletion with a delay after commit.
     * This is necessary as long as the MDS is validating the artifact was deleted before deleting it.
     */
    @Async(delayUntilAfterCommit = true)
    void submitPackageDeletionAfterCommit(RepoPath repoPath);
}
