/*
 *
 * Artifactory is a binaries repository manager.
 * Copyright (C) 2018 JFrog Ltd.
 *
 * Artifactory is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * Artifactory is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Artifactory.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.artifactory.repo.interceptor.config;

import org.apache.commons.lang.StringUtils;
import org.artifactory.api.config.RepositoryConfigService;
import org.artifactory.common.crypto.CryptoHelper;
import org.artifactory.common.home.ArtifactoryHome;
import org.artifactory.model.RemoteRepoConfig;
import org.artifactory.repo.RepoConfigChange;
import org.artifactory.repo.RepoDetailsType;
import org.artifactory.repo.interceptor.RepoConfigChangeInterceptor;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;

import static org.artifactory.layout.EncryptConfigurationInterceptor.getNewPassword;
import static org.artifactory.repo.RepoConfigChange.ConfigAction.REPO_CREATE;
import static org.artifactory.repo.RepoConfigChange.ConfigAction.REPO_UPDATE;

/**
 * @author Fred Simon
 */
@Component
public class EncryptRepoConfigInterceptor implements RepoConfigChangeInterceptor {
    @Override
    public void onBeforeSave(Map<RepoDetailsType, List<RepoConfigChange>> repoConfigChanges) {
        if (CryptoHelper.hasArtifactoryKey(ArtifactoryHome.get())) {
            // Find all sensitive data and encrypt them
            List<RemoteRepoConfig> newRemoteRepoConfigs =
                    repoConfigChanges.getOrDefault(RepoDetailsType.REMOTE, List.of()).stream()
                            .filter(repoConfigChange -> REPO_UPDATE.equals(repoConfigChange.getConfigAction()) ||
                                    REPO_CREATE.equals(repoConfigChange.getConfigAction()))
                            .map(RepoConfigChange::getNewRepoConfig)
                            .map(RemoteRepoConfig.class::cast)
                            .toList();
            encrypt(newRemoteRepoConfigs);
        }
    }

    public static void decrypt(RepositoryConfigService repositoryConfigService) {
        List<RemoteRepoConfig> remoteRepoConfigs =
                repositoryConfigService.findRepositoriesByType(RepoDetailsType.REMOTE).stream()
                        .map(RemoteRepoConfig.class::cast)
                        .toList();
        encryptOrDecrypt(remoteRepoConfigs, false);
        repositoryConfigService.bulkUpsertRepositoryConfigs(remoteRepoConfigs, true);
    }

    private static void encryptOrDecrypt(List<RemoteRepoConfig> newRemoteRepoConfigs,
            boolean encrypt) {
        for (RemoteRepoConfig remoteRepoConfig : newRemoteRepoConfigs) {
            String newPassword = getNewPassword(encrypt, remoteRepoConfig.getRepoTypeConfig().getPassword());
            if (StringUtils.isNotBlank(newPassword)) {
                remoteRepoConfig.getRepoTypeConfig().setPassword(newPassword);
            }
        }
    }

    private void encrypt(List<RemoteRepoConfig> newRemoteRepoConfigs) {
        encryptOrDecrypt(newRemoteRepoConfigs, true);
    }
}
