package org.artifactory.build;

import lombok.val;
import org.artifactory.api.config.ImportSettingsImpl;
import org.artifactory.common.MutableStatusHolder;
import org.artifactory.sapi.common.ImportKind;
import org.artifactory.sapi.common.ImportSettings;
import org.junit.jupiter.api.Test;

import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertTrue;

class BuildServiceHybridMigrationTest {

    @Test
    void whenHybridMigrationThenBuildAndReposAreNotDeleted() {
        val buildService = new BuildServiceForTest();
        val importSettings = new ImportSettingsImpl(null);
        importSettings.setImportKind(ImportKind.HYBRID_MIGRATION);
        buildService.importFrom(importSettings);
        assertFalse(buildService.buildsAndReposAreDeleted);
    }

    @Test
    void whenStandardImportThenBuildAndReposAreDeleted() {
        val buildService = new BuildServiceForTest();
        val importSettings = new ImportSettingsImpl(null);
        importSettings.setImportKind(ImportKind.STANDARD);
        buildService.importFrom(importSettings);
        assertTrue(buildService.buildsAndReposAreDeleted);
    }

    class BuildServiceForTest extends BuildServiceImpl {

        private boolean buildsAndReposAreDeleted = false;

        @Override
        void clearCaches() {

        }

        @Override
        void deleteBuildsAndRepositories(MutableStatusHolder status) {
            buildsAndReposAreDeleted = true;
        }

        @Override
        void importBuildFiles(ImportSettings settings, MutableStatusHolder status) {

        }
    }
}