package org.artifactory.hqc.config;

import org.artifactory.test.ArtifactoryHomeBoundJUnitTest;
import org.artifactory.test.ArtifactoryHomeStub;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import static org.artifactory.common.ConstantValues.hqcExpiryFactor;
import static org.artifactory.common.ConstantValues.hqcMaxBlobSizeKb;
import static org.artifactory.common.ConstantValues.hqcMinCallableDurationMillis;
import static org.junit.jupiter.api.Assertions.assertEquals;

/**
 * @author Andrey Tashlitsky
 */
class HeavyQueryCacheConfigTest extends ArtifactoryHomeBoundJUnitTest {

    private final ArtifactoryHomeStub home = getOrCreateArtifactoryHomeStub();

    @BeforeEach
    public void beforeEach() {
        home.setProperty(hqcMinCallableDurationMillis.getPropertyName(), hqcMinCallableDurationMillis.getDefValue());
        home.setProperty(hqcMaxBlobSizeKb.getPropertyName(), hqcMaxBlobSizeKb.getDefValue());
        home.setProperty(hqcExpiryFactor.getPropertyName(), hqcExpiryFactor.getDefValue());
    }

    @Test
    void testDefaultValues() {
        HeavyQueryCacheConfig hqcConfig = HeavyQueryCacheConfig.builder().build();
        assertEquals(hqcMinCallableDurationMillis.getLong(), hqcConfig.getMinCallableDurationMillis());
        assertEquals(hqcMaxBlobSizeKb.getInt(), hqcConfig.getMaxBlobSizeKb());
        assertEquals(hqcExpiryFactor.getInt(), hqcConfig.getExpiryFactor());
        assertEquals(Integer.MIN_VALUE, hqcConfig.getExpiryLowerBound());
        assertEquals(Integer.MAX_VALUE, hqcConfig.getExpiryUpperBound());
    }

    @Test
    void testNonDefaultValues() {
        HeavyQueryCacheConfig hqcConfig = HeavyQueryCacheConfig.builder()
                .minCallableDurationMillis(1234)
                .maxBlobSizeKb(7)
                .expiryFactor(34)
                .expiryLowerBound(1289)
                .expiryUpperBound(2378)
                .build();
        assertEquals(1234, hqcConfig.getMinCallableDurationMillis());
        assertEquals(7, hqcConfig.getMaxBlobSizeKb());
        assertEquals(34, hqcConfig.getExpiryFactor());
        assertEquals(1289, hqcConfig.getExpiryLowerBound());
        assertEquals(2378, hqcConfig.getExpiryUpperBound());
    }

    @Test
    void testOverridingConstantValues() {
        home.setProperty(hqcMinCallableDurationMillis.getPropertyName(), "101");
        home.setProperty(hqcMaxBlobSizeKb.getPropertyName(), "3");
        home.setProperty(hqcExpiryFactor.getPropertyName(), "7");
        HeavyQueryCacheConfig hqcConfig = HeavyQueryCacheConfig.builder().build();
        assertEquals(101, hqcConfig.getMinCallableDurationMillis());
        assertEquals(3, hqcConfig.getMaxBlobSizeKb());
        assertEquals(7, hqcConfig.getExpiryFactor());
        assertEquals(Integer.MIN_VALUE, hqcConfig.getExpiryLowerBound());
        assertEquals(Integer.MAX_VALUE, hqcConfig.getExpiryUpperBound());
    }
}
