package org.artifactory.hqc.service;

import lombok.Getter;
import org.artifactory.storage.db.hqc.dao.HeavyQueryCacheDao;
import org.artifactory.storage.db.hqc.entity.HeavyQueryCacheEntry;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicLong;

import static org.artifactory.common.ConstantValues.dbOperationsBulkSize;

/**
 * @author Tomer Slutsky
 * @author Andrey Tashlitsky
 */
public class HeavyQueryCacheDaoMock extends HeavyQueryCacheDao {

    private static final AtomicLong id = new AtomicLong(0);
    @Getter
    private final Map<Long, HeavyQueryCacheEntry> hqcMap = new HashMap<>();

    public HeavyQueryCacheDaoMock() {
        super(null, null);
    }

    @Override
    public HeavyQueryCacheEntry getCachedResponse(String cacheKey, long time) {
        return hqcMap.values().stream()
                .filter(e -> e.getCacheKey().equals(cacheKey) && e.getExpiryTime() > time)
                .findFirst()
                .orElse(null);
    }

    @Override
    public List<HeavyQueryCacheEntry> getExpiredEntries(long time) {
        return hqcMap.values().stream()
                .filter(entry -> entry.getExpiryTime() < time)
                .limit(dbOperationsBulkSize.getLong())
                .toList();
    }

    @Override
    public void delete(List<HeavyQueryCacheEntry> cacheEntries) {
        cacheEntries.stream()
                .map(HeavyQueryCacheEntry::getId)
                .forEach(e -> hqcMap.values().removeIf(entry -> entry.getId() == e));
    }

    @Override
    public void delete(HeavyQueryCacheEntry cacheEntry) {
        hqcMap.values().removeIf(entry -> entry.getCacheKey().equals(cacheEntry.getCacheKey()));
    }

    @Override
    public int create(HeavyQueryCacheEntry cacheEntry) {
        hqcMap.put(id.incrementAndGet(), cacheEntry);
        return 1;
    }
}
