package org.artifactory.repo.service.validation;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.MethodSource;
import org.mockito.MockitoAnnotations;

import static org.junit.jupiter.api.Assertions.assertThrows;

class RepositoryNameYamlConfigurationValidatorTest {

    private AutoCloseable mockitoAutoCloseable;

    @BeforeEach
    void setUp() {
        mockitoAutoCloseable = MockitoAnnotations.openMocks(this);
    }

    @AfterEach
    void tearDown() throws Exception {
        mockitoAutoCloseable.close();
    }

    @ParameterizedTest
    @MethodSource("repoNames")
    void testInvalidRepoName(String repoName) {
        assertThrows(IllegalArgumentException.class,
                () -> RepositoryNameYamlConfigurationValidator.validateRepoName(repoName, "local"));
    }

    private static Object[][] repoNames() {
        return new Object[][] {
                {"jfrog-artifactory-system-repo-name"},
                {"jfrog-system-reserved-repo-name"},
                {"some-repo-?-name"},
                {"/some-repo-name"},
                {"some-repo-name*"},
                {""},
                {null},
                {"very-very-l0000000oooooooooooooooooooooooooooooooooooong-local-repo"},
                {"_intransit"},
                {"artifactory-pipe-info"}
        };
    }

}
