package org.artifactory.addon.curation;

import org.apache.http.HttpRequestInterceptor;
import org.artifactory.addon.Addon;
import org.artifactory.common.StatusHolder;
import org.artifactory.model.RemoteRepoConfig;
import org.artifactory.repo.RepoPath;

import javax.ws.rs.core.Response;
import java.util.List;
import java.util.Optional;

/**
 * @author Dvir Twina
 * @author Tomer Slutsky
 */
public interface CurationAddon extends Addon {

    boolean isCurationAsService();

    /**
     * Toggles the curation enabled/disabled flag in the curation settings
     *
     * @param body - Enable/disable the curation
     * @return If the operation was successful
     */
    Response toggleCuration(ToggleCuration body);

    /**
     * Gets the curation service trust token if it exists
     *
     * @return The service trust token as json
     */
    Response getToken();

    void refreshMasterToken();

    /**
     * Gets the curation interceptor if the service trust is enabled
     *
     * @return The interceptor to create short lived tokens
     */
    HttpRequestInterceptor getCurationRequestInterceptor();

    String generateNamespace();

    String getNamespace();

    /**
     * Gets the JPD paired access token from the curation settings
     *
     * @return The paired access token
     */
    Optional<String> getPairedToken();

    /**
     * Gets the URL of the paired JPD instance from the curation settings
     *
     * @return The paired base url
     */
    Optional<String> getPairedInstance();

    void saveSettings(String settings);

    Optional<String> getCurationSettings();

    /**
     * Checks if we are using service trust mode for phase 2 of curation
     *
     * @return If the feature flag for service trust curation is on
     */
    boolean isCurationWithTrustEnabled();

    boolean isUserAllowed(String currentUsername);

    boolean isCurationEntitled();

    boolean isCurationEnabled();

    boolean isCurationConfigValid(RemoteRepoConfig config);

    List<CurationSupportedPackageType> getSupportedPackageTypes();

    List<CurationSupportedRepository> getSupportedRepositories();

    StatusHolder getPackageStatus(RemoteRepoConfig config, RepoPath path);
}
