package org.artifactory.hqc.annotation;

import org.artifactory.hqc.enums.HeavyQueryCacheType;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Inherited;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * Use this to apply the Heavy Query Cache mechanism on a service method.<br>
 * <b>NOTE: Make sure that each of the parameters of the annotated method override {@link Object#toString()}. This
 * is due to the fact that HeavyQueryCacheAdvice#invoke use the parameters' string representation to create a unique
 * name. Also, make sure the resulting string is not too long for the distributed_locks#lock_key column.</b>
 *
 * @author Andrey Tashlitsky
 */
@Target({ElementType.METHOD})
@Retention(RetentionPolicy.RUNTIME)
@Inherited
@Documented
public @interface HeavyQueryCache {

    HeavyQueryCacheType hqcType();

    int minCallableDurationMillis() default -1;

    int maxBlobSizeKb() default -1;

    int expiryFactor() default -1;

    int expiryLowerBound() default -1;

    int expiryUpperBound() default -1;
}
