package org.artifactory.lifecycle.converter.postinit.v105;

import org.artifactory.api.context.ArtifactoryContext;
import org.artifactory.api.context.ArtifactoryContextThreadBinder;
import org.artifactory.common.home.ArtifactoryHome;
import org.artifactory.converter.ConverterPreconditionException;
import org.artifactory.converter.mirror.MetadataNegotiationService;
import org.artifactory.version.CompoundVersionDetails;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.Test;

import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;
import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertFalse;
import static org.testng.Assert.assertTrue;
import static org.testng.Assert.fail;

/**
 * @author Alexei Vainshtein
 */
public class FederationRepoNegotiationConverterTest {

    @AfterMethod
    public void tearDown() {
        ArtifactoryHome.unbind();
        ArtifactoryContextThreadBinder.unbind();
    }

    @Test
    public void testConvert() {
        ArtifactoryContext context = mock(ArtifactoryContext.class);
        ArtifactoryContextThreadBinder.bind(context);
        MetadataNegotiationService metadataNegotiationService = mock(MetadataNegotiationService.class);
        when(context.beanForType(MetadataNegotiationService.class)).thenReturn(metadataNegotiationService);
        CompoundVersionDetails source = mock(CompoundVersionDetails.class);
        FederationRepoNegotiationConverter federationRepoNegotiationConverter =
                new FederationRepoNegotiationConverter(null, null);
        federationRepoNegotiationConverter.convert(source, source);
        verify(metadataNegotiationService).negotiateUponUpgrade(source);
    }

    @Test
    public void testAssertConversionPrecondition() {
        ArtifactoryHome home = mock(ArtifactoryHome.class);
        ArtifactoryHome.bind(home);
        FederationRepoNegotiationConverter federationRepoNegotiationConverter =
                new FederationRepoNegotiationConverter(null, null);
        federationRepoNegotiationConverter.assertConversionPrecondition(null, null, null);

        ArtifactoryHome.unbind();
        try {
            federationRepoNegotiationConverter.assertConversionPrecondition(null, null, null);
            fail("Expected to fail");
        } catch (ConverterPreconditionException e) {
            assertEquals(e.getMessage(), "Artifactory home is missing");
        }
    }

    @Test
    public void testIsInterested() {
        FederationRepoNegotiationConverter federationRepoNegotiationConverter =
                new FederationRepoNegotiationConverter(null, null);
        CompoundVersionDetails source = mock(CompoundVersionDetails.class);
        when(source.isCurrent()).thenReturn(false);
        CompoundVersionDetails version = mock(CompoundVersionDetails.class);
        assertTrue(federationRepoNegotiationConverter.isInterested(source, version));

        when(source.isCurrent()).thenReturn(true);
        assertFalse(federationRepoNegotiationConverter.isInterested(source, version));

        assertFalse(federationRepoNegotiationConverter.isInterested(null, version));
    }
}