package org.artifactory.logging.filters;

import ch.qos.logback.core.spi.FilterReply;
import com.jfrog.sysconf.SysConfig;
import org.artifactory.api.context.ArtifactoryContextThreadBinder;
import org.artifactory.common.home.ArtifactoryHome;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.Arguments;
import org.junit.jupiter.params.provider.MethodSource;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;

import java.util.stream.Stream;

import static org.artifactory.logging.filters.ConsoleJsonFormatFilter.ENABLE_CONSOLE_LOG_PRO_NAME;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

/**
 * @author Tomer Slutsky
 */
public class ConsoleJsonFormatFilterTest {
    @Mock
    ArtifactoryHome home;

    ConsoleJsonFormatFilter consoleJsonFormatFilter;
    AutoCloseable closeable;

    static Stream<Arguments> provider() {
        return Stream.of(
                Arguments.of(false, FilterReply.DENY),
                Arguments.of(true, FilterReply.ACCEPT)
        );
    }

    @BeforeEach
    public void setUp() {
        closeable = MockitoAnnotations.openMocks(this);
        ArtifactoryHome.bind(home);
        consoleJsonFormatFilter = new ConsoleJsonFormatFilter();
    }

    @AfterEach
    public void tearDown() throws Exception {
        closeable.close();
        ArtifactoryHome.unbind();
        ArtifactoryContextThreadBinder.unbind();
    }

    @ParameterizedTest
    @MethodSource("provider")
    public void testDecide(boolean found, FilterReply expectedReply) {
        SysConfig sysConfig = mock(SysConfig.class);
        when(ArtifactoryHome.get().getSysConfig()).thenReturn(sysConfig);
        when(sysConfig.getBoolean(ENABLE_CONSOLE_LOG_PRO_NAME, false)).thenReturn(found);
        Assertions.assertEquals(consoleJsonFormatFilter.decide(null), expectedReply);
    }
}