package org.artifactory.descriptor.repo;

import lombok.AccessLevel;
import lombok.Getter;
import lombok.Setter;
import org.artifactory.descriptor.Descriptor;
import org.jfrog.common.config.diff.DiffKey;

import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlID;
import javax.xml.bind.annotation.XmlType;
import java.util.Objects;

@XmlType(name = "FederatedMembersType", propOrder = {"url", "mode", "enabled"},
        namespace = Descriptor.NS)
@Getter
@Setter
public class FederatedMember implements Descriptor {

    @XmlID
    @XmlElement(required = true)
    @Getter(value = AccessLevel.NONE)
    private String url;
    private String mode;

    @XmlElement(defaultValue = "true", required = true)
    @DiffKey
    private boolean enabled = true;

    @Override
    public int hashCode() {
        return Objects.hash(url, enabled);
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }
        FederatedMember that = (FederatedMember) o;
        if (enabled != that.enabled) {
            return false;
        }

        return Objects.equals(url, that.url);
    }

    @Override
    public String toString() {
        return "FederatedMember{" +
                "url='" + url + '\'' +
                ", enabled=" + enabled +
                '}';
    }

    /**
     * Returns the federated member URL that may or may not contain a baseURL placeholder (template), depending on
     * the type of federated member (remote, local)
     * The placeholder URL that is returned from this method will be resolved dynamically, rather than containing a
     * strict base URL string.
     *
     * @return the full federated member URL for remote members, and a URL template for the local member.
     */
    public String getUrlOrTemplate() {
        return url;
    }

    public void setUrlOrTemplate(String url) {
        this.url = url;
    }
}
