package org.artifactory.model;

import lombok.experimental.UtilityClass;

import java.util.Objects;

/**
 * @author Dor Solomon
 */
@UtilityClass
public class RepoConfigCopyFactory {

    /**
     * Create a deep copy of repo config
     *
     * @param repoConfig the repo config to be copied
     * @return a copy of the repo config
     * @throws NullPointerException in case {@code repoConfig} is null
     */
    public RepoConfig copy(RepoConfig repoConfig) {
        Objects.requireNonNull(repoConfig, "Repo config cannot be null");
        if (repoConfig instanceof ReleaseBundleRepoConfig releaseBundleRepoConfig) {
            return ReleaseBundleRepoConfig.copy(releaseBundleRepoConfig);
        }
        if (repoConfig instanceof FederatedRepoConfig federatedRepoConfig) {
            return FederatedRepoConfig.copy(federatedRepoConfig);
        }
        if (repoConfig instanceof LocalRepoConfig localRepoConfig) {
            return LocalRepoConfig.copy(localRepoConfig);
        }
        if (repoConfig instanceof RemoteRepoConfig remoteRepoConfig) {
            return RemoteRepoConfig.copy(remoteRepoConfig);
        }
        if (repoConfig instanceof VirtualRepoConfig virtualRepoConfig) {
            return VirtualRepoConfig.copy(virtualRepoConfig);
        }
        return null;
    }
}
