/*
 *
 * Artifactory is a binaries repository manager.
 * Copyright (C) 2018 JFrog Ltd.
 *
 * Artifactory is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * Artifactory is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Artifactory.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.artifactory.storage.db.binstore.service;

import org.artifactory.api.blob.infos.BlobInfosDBService;
import org.artifactory.api.context.ContextHelper;
import org.jfrog.storage.events.DeletedBinariesEvent;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationListener;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @author Inbar Tal
 */
@Component
public class DeleteBlobInfosWrapperImpl implements ApplicationListener<DeletedBinariesEvent>, DeleteBlobInfosWrapper {

    private BlobInfosDBService blobInfosDBService;

    @Override
    public void deleteBlobInfosAsync(List<String> checksums) {
        blobInfosDBService.deleteBlobInfos(checksums);
    }

    @Override
    public void onApplicationEvent(DeletedBinariesEvent event) {
        // Using getMe() for having the @Async
        getMe().deleteBlobInfosAsync(event.getDeletedBinaries());
    }

    private DeleteBlobInfosWrapper getMe() {
        return ContextHelper.get().beanForType(DeleteBlobInfosWrapper.class);
    }

    @Autowired
    public void setBlobInfosDBService(BlobInfosDBService blobInfosDBService) {
        this.blobInfosDBService = blobInfosDBService;
    }
}
