package org.artifactory.storage.db.converter.markers;

import org.artifactory.converter.ArtifactoryConverterAdapter;
import org.artifactory.converter.ConverterType;
import org.artifactory.test.ArtifactoryHomeBoundTest;
import org.artifactory.version.CompoundVersionDetails;
import org.artifactory.version.VersionProvider;
import org.jfrog.config.ifc.ConfigurationManager;
import org.mockito.Mockito;
import org.testng.Assert;
import org.testng.annotations.Test;

import java.util.ArrayList;
import java.util.Map;

/**
 * @author Alexei Vainshtein
 */
public class ConverterManagerImplTest extends ArtifactoryHomeBoundTest {

    @Test
    public void testServiceConvertAsync() {
        ConfigurationManager configurationManager = Mockito.mock(ConfigurationManager.class);
        VersionProvider vp = Mockito.mock(VersionProvider.class);
        CompoundVersionDetails compoundVersionDetails = Mockito.mock(CompoundVersionDetails.class);
        Mockito.when(compoundVersionDetails.isCurrent()).thenReturn(true);
        Mockito.when(vp.getOriginalDbVersion()).thenReturn(compoundVersionDetails);
        ConverterManagerImpl converterManager =
                Mockito.spy(new ConverterManagerImpl(homeStub, vp, configurationManager));
        Map<ConverterType, ArrayList<ArtifactoryConverterAdapter>> converters = converterManager.getConverters();
        converters.get(ConverterType.ASYNC).removeAll(converters.get(ConverterType.ASYNC));

        ArtifactoryConverterAdapter converterAdapter = Mockito.mock(ArtifactoryConverterAdapter.class);
        converters.get(ConverterType.ASYNC).add(converterAdapter);

        converterManager.convertAsync();
        Mockito.verify(converterAdapter, Mockito.times(1)).convert(Mockito.any(), Mockito.any());
        Mockito.verify(converterManager, Mockito.times(0)).shouldBlockConvert();

        Mockito.clearInvocations(converterAdapter);
        Mockito.when(compoundVersionDetails.isCurrent()).thenReturn(false);
        converterManager.convertAsync();
        Mockito.verify(converterAdapter, Mockito.times(1)).convert(Mockito.any(), Mockito.any());
        Mockito.verify(converterManager, Mockito.times(1)).shouldBlockConvert();
    }

    @Test
    public void testInit() {
        ConfigurationManager configurationManager = Mockito.mock(ConfigurationManager.class);
        VersionProvider vp = Mockito.mock(VersionProvider.class);
        ConverterManagerImpl converterManager = new ConverterManagerImpl(homeStub, vp, configurationManager);
        Map<ConverterType, ArrayList<ArtifactoryConverterAdapter>> converters = converterManager.getConverters();
        Assert.assertNotNull(converters.get(ConverterType.ASYNC));
    }
}