package org.artifactory.ui.rest.resource.utils.repositories;

import com.google.common.annotations.VisibleForTesting;
import org.artifactory.api.security.AuthorizationService;
import org.artifactory.rest.common.resource.BaseResource;
import org.artifactory.rest.common.service.RestService;
import org.artifactory.rest.exception.AuthorizationRestException;
import org.artifactory.ui.rest.service.utils.UtilsServiceFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import javax.annotation.security.RolesAllowed;
import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;

import static org.artifactory.ui.rest.resource.utils.UtilsResourcesConstants.PATH_PAGED_REPODATA;

/**
 * @author Dvir Twina
 */
@RolesAllowed({AuthorizationService.ROLE_ADMIN, AuthorizationService.ROLE_USER})
@Component
@Path(PATH_PAGED_REPODATA)
@Scope(BeanDefinition.SCOPE_PROTOTYPE)
public class PagedRepositoriesDataResource extends BaseResource {

    private UtilsServiceFactory utilsFactory;
    private AuthorizationService authorizationService;

    @Autowired
    public void setUtilsFactory(UtilsServiceFactory utilsFactory) {
        this.utilsFactory = utilsFactory;
    }

    @Autowired
    public void setAuthorizationService(AuthorizationService authorizationService) {
        this.authorizationService = authorizationService;
    }

    /**
     * Gets the list of local, remote and federated (according to params) repos for the Permission Repositories page.
     */
    @GET
    @Produces(MediaType.APPLICATION_JSON)
    public Response getPagedRepoData() {
        if (!authorizationService.isAdmin() && !authorizationService.isResourceManager()) {
            throw new AuthorizationRestException();
        }
        return runService(utilsFactory.getAllRepositoriesPagedService());
    }

    @Override
    @VisibleForTesting
    protected <Y> Response runService(RestService<Y> service) {
        return super.runService(service);
    }
}
