package org.artifactory.ui.rest.service.admin.configuration.mail;

import org.artifactory.addon.AddonsManager;
import org.artifactory.aol.AolService;
import org.artifactory.api.context.ArtifactoryContext;
import org.artifactory.api.context.ArtifactoryContextThreadBinder;
import org.artifactory.rest.common.service.ArtifactoryRestRequest;
import org.artifactory.rest.common.service.ArtifactoryRestResponse;
import org.artifactory.rest.common.service.RestResponse;
import org.artifactory.ui.rest.model.admin.configuration.mail.MailServer;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;

import static org.artifactory.ui.utils.RequestUtils.PASSWORD_PLACEHOLDER;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.spy;
import static org.mockito.Mockito.when;

class UpdateMailServiceTest {

    UpdateMailService updateMailService;

    @BeforeEach
    void setup() {
        updateMailService = spy(new UpdateMailService());
        ArtifactoryContext artifactoryContext = mock(ArtifactoryContext.class);
        AddonsManager addonsManagerMock = mock(AddonsManager.class);
        AolService aolService = mock(AolService.class);

        when(artifactoryContext.beanForType(AddonsManager.class)).thenReturn(addonsManagerMock);

        when(artifactoryContext.beanForType(AolService.class)).thenReturn(aolService);
        when(aolService.isAol()).thenReturn(false);

        ArtifactoryContextThreadBinder.bind(artifactoryContext);
    }

    @AfterEach
    void cleanup() {
        ArtifactoryContextThreadBinder.unbind();
    }

    @Test
    void testUpdateWithPasswordPlaceHolder() {
        ArtifactoryRestRequest artifactoryRestRequest = Mockito.mock(ArtifactoryRestRequest.class);
        MailServer mailServer = Mockito.mock(MailServer.class);
        when(mailServer.getPassword()).thenReturn(PASSWORD_PLACEHOLDER);
        when(artifactoryRestRequest.getImodel()).thenReturn(mailServer);
        RestResponse artifactoryRestResponse = new ArtifactoryRestResponse();
        updateMailService.execute(artifactoryRestRequest, artifactoryRestResponse);
        Assertions.assertEquals(
                "{\"error\":\"Mail server password needs to be re-entered to allow changes in Mail server settings.\"}",
                artifactoryRestResponse.getEntity());
    }
}