package org.artifactory.ui.rest.service.admin.configuration.repositories.util;

import org.artifactory.addon.build.BuildAddon;
import org.artifactory.build.InternalBuildService;
import org.artifactory.federation.FederatedRepoService;
import org.artifactory.repo.system.SystemRepos;
import org.artifactory.rest.common.service.ArtifactoryRestRequest;
import org.artifactory.rest.common.service.ArtifactoryRestResponse;
import org.artifactory.rest.common.service.RestResponse;
import org.artifactory.ui.rest.model.admin.configuration.repository.federation.FederatedBuildInfoDetails;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;

import static org.junit.jupiter.api.Assertions.assertTrue;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;
import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertFalse;

class GetFederatedBuildInfoDetailsServiceTest {

    @Mock
    FederatedRepoService federatedRepoService;
    @Mock
    InternalBuildService buildService;

    @InjectMocks
    GetFederatedBuildInfoDetailsService service;

    @BeforeEach
    public void init() {
        MockitoAnnotations.openMocks(this);
    }


    @Test
    void getFederatedBuildInfo() {
        ArtifactoryRestRequest<String> request = mock(ArtifactoryRestRequest.class);
        RestResponse<FederatedBuildInfoDetails> response = new ArtifactoryRestResponse();

        when(request.getImodel()).thenReturn(null);
        when(federatedRepoService.isProjectBuildInfoRepoFederated(null))
                .thenReturn(true);
        when(buildService.getBuildInfoRepoKeyByProject(null)).thenReturn(SystemRepos.BUILDINFO_REPO_KEY);
        service.execute(request, response);

        assertFalse(((FederatedBuildInfoDetails) response.getIModel()).isEnableConvert());
        assertEquals(((FederatedBuildInfoDetails) response.getIModel()).getBuildInfoRepo(),
                SystemRepos.BUILDINFO_REPO_KEY);
    }

    @Test
    void testProjectBuildInfoRepo() {
        ArtifactoryRestRequest<String> request = mock(ArtifactoryRestRequest.class);
        RestResponse<FederatedBuildInfoDetails> response = new ArtifactoryRestResponse();
        String projectKey = "proja";
        String projectRepo = projectKey + BuildAddon.BUILD_INFO_REPO_EXTENSION;
        when(request.getImodel()).thenReturn(projectKey);
        when(federatedRepoService.isProjectBuildInfoRepoFederated(projectKey))
                .thenReturn(false);
        when(buildService.getBuildInfoRepoKeyByProject(projectKey)).thenReturn(projectRepo);
        service.execute(request, response);

        assertTrue(((FederatedBuildInfoDetails) response.getIModel()).isEnableConvert());
        assertEquals(((FederatedBuildInfoDetails) response.getIModel()).getBuildInfoRepo(),
                projectRepo);
    }
}